// Copyright 2017-2025 WIT.COM Inc. All rights reserved.
// Use of this source code is governed by the GPL 3.0

// 2025 note by jcarr:
// this is one of the coolest things ever worked with.
// Personally, I've been working on making a gocui GO plugin
// so I can use it as a generalized console GUI toolkit.
//
// Well done everyone that has contributed to this gocui project !!!
// I am in your debt. Happy hacking & peace.

package main

import (
	"fmt"

	"github.com/awesome-gocui/gocui"
	log "go.wit.com/log"
	"go.wit.com/widget"
)

// this function uses the mouse position to highlight & unhighlight things
// this is run every time the user moves the mouse over the terminal window
func mouseMove(g *gocui.Gui) {
	mx, my := g.MousePosition()

	w := mx
	h := my

	if me.supermouse {
		for _, tk := range findByXY(w, h) {
			tk.dumpWidget("mouseMove()")
		}
	}

	if me.globalMouseDown {
		// log.Info("msgMouseDown == true")
		// plugin will segfault if you don't keep this inside a check for msgMouseDown
		// don't move this code out of here
		var found bool = false
		for _, tk := range findByXY(w, h) {
			if tk.node.WidgetType == widget.Stdout {
				tk.moveNew(g)
				return
			}
			if tk.node.WidgetType == widget.Label {
				tk.moveNew(g)
				return
			}
			found = true
		}
		if !found {
			log.Info(fmt.Sprintf("findByXY() empty. nothing to move at (%d,%d)", w, h))
		}
	}

	if widgetView, _ := g.View("msg"); widgetView == nil {
		if createStdout(g) {
			return
		}
		return
	}

	for _, view := range g.Views() {
		view.Highlight = false
	}
	if v, err := g.ViewByPosition(mx, my); err == nil {
		v.Highlight = true
	}
}

// this is how the window gets dragged around
func (tk *guiWidget) moveNew(g *gocui.Gui) {
	w, h := g.MousePosition()
	if tk.node.WidgetType == widget.Label {
		s := fmt.Sprintf("move(%dx%d) %s ###", w, h, tk.cuiName)
		tk.dumpWidget(s)
		outputW, outputH := tk.Size()
		g.SetView(tk.cuiName, w-xOffset, h-yOffset, w-xOffset+outputW, h-yOffset+outputH+me.FramePadH, 0)
		me.startOutputW = w - xOffset
		me.startOutputH = h - yOffset
		// g.SetViewOnBottom(tk.cuiName)
		return
	}
	tk.dumpWidget("moveNew() on " + tk.cuiName)
	outputW := 140
	outputH := 40
	g.SetView("msg", w-xOffset, h-yOffset, w-xOffset+outputW, h-yOffset+outputH+me.FramePadH, 0)
	me.startOutputW = w - xOffset
	me.startOutputH = h - yOffset
	g.SetViewOnBottom("msg")
}

func createStdout(g *gocui.Gui) bool {
	makeOutputWidget(g, "this is a create before a mouse click")
	if me.logStdout != nil {
		msg := fmt.Sprintf("test out gocuiEvent() %d\n", me.ecount)
		// me.logStdout.v.Write([]byte(msg))
		me.logStdout.Write([]byte(msg))
		log.Log(NOW, "logStdout test out")
	}
	return true
}
